@@ -5,7 +5,7 @@
 #
 # See below for version information
 #
-# Copyright (c) 2020 Windell H. Oskay, Evil Mad Scientist Laboratories
+# Copyright (c) 2021 Windell H. Oskay, Evil Mad Scientist Laboratories
 #
 # The MIT License (MIT)
 #
@@ -47,7 +47,7 @@ ffgeom = from_dependency_import('ink_extensions.ffgeom')
 
 def version():    # Version number for this document
     """Return version number of this script"""
-    return "0.18" # Dated 2020-09-29
+    return "0.21" # Dated 2021-10-13
 
 __version__ = version()
 
@@ -99,6 +99,26 @@ def checkLimitsTol(value, lower_bound, upper_bound, tolerance):
     return value, False  # Return original value without error
 
 
+def point_in_bounds(point, bounds, tolerance=1e-9):
+    """
+    Given an input point [x, y], a rectangular bounding region
+    [[x_min, y_min], [x_max, y_max]], and an optional tolerance,
+    return True if the point is within the bounds, to within the
+    required tolerance, and False otherwise.
+    """
+    x, y = point
+    [[x_min, y_min], [x_max, y_max]] = bounds
+    if x < x_min - tolerance:
+        return False
+    if y < y_min - tolerance:
+        return False
+    if x > x_max + tolerance:
+        return False
+    if y > y_max + tolerance:
+        return False
+    return True
+
+
 def clip_code(x_in, y_in, x_min, x_max, y_min, y_max):
     """Encode point position with respect to boundary box"""
     code = 0
@@ -135,6 +155,8 @@ def clip_segment(segment, bounds):
     x_max = bounds[1][0]
     y_max = bounds[1][1]
 
+    iterations = 0
+
     while True: # Repeat until return
         code_1 = clip_code(x_1, y_1, x_min, x_max, y_min, y_max)
         code_2 = clip_code(x_2, y_2, x_min, x_max, y_min, y_max)
@@ -145,6 +167,8 @@ def clip_segment(segment, bounds):
         # Trivial reject, if both endpoints are outside, and on the same side:
         if code_1 & code_2:
             return False, segment # Verify with bitwise AND.
+        if iterations > 3: # Failsafe; exit if the value has not converged;
+            return True, segment # Avoids infinite loops near precision limits.
 
         # Otherwise, at least one point is out of bounds; not trivial.
         if code_1 != 0:
@@ -181,6 +205,7 @@ def clip_segment(segment, bounds):
             x_2 = x_new
             y_2 = y_new
         segment = [[x_1, y_1], [x_2, y_2]] # Now checking this clipped segment
+        iterations += 1
 
 
 def constrainLimits(value, lower_bound, upper_bound):
@@ -350,6 +375,21 @@ def unitsToUserUnits(input_string):
     return None
 
 
+def position_scale (x_value, y_value, units_code):
+    '''
+    Format XY position data to be returned to user
+    x_value, y_value inputs are in inches.
+    Output set by units_code: 1 for cm, 2 for mm, 0 (or otherwise) for inch.
+    '''
+    if units_code == 1 : # If using centimeter units
+        x_value = x_value * 2.54
+        y_value = y_value * 2.54
+    if units_code == 2: # If using millimeter units
+        x_value = x_value * 25.4
+        y_value = y_value * 25.4
+    return x_value, y_value
+
+
 def subdivideCubicPath(s_p, flat, i=1):
     """
     Break up a bezier curve into smaller curves, each of which
@@ -641,34 +681,6 @@ def square_dist(point_a, point_b):
     return delta_x * delta_x + delta_y * delta_y
 
 
-def points_near(point_a, point_b, squared_tolerance):
-    """
-    Given two vertices point_a and point_b, each a 2-tuple, return True if the two
-    points are coincident to within a certain tolerance.
-
-    Arguments:
-        point_a, point_b:  Vertex (x,y), 2-tuples of floats
-        squared_tolerance: Square of maximum allowed distance between vertices
-
-    if (point_a.x - point_b.x)^2 + (point_a.y - point_b.y)^2 < tolerance^2,
-        then return True.
-    """
-    delta_x = point_a[0] - point_b[0]
-    delta_y = point_a[1] - point_b[1]
-
-    return (delta_x * delta_x + delta_y * delta_y) < squared_tolerance
-
-
-def square_dist(point_a, point_b):
-    """
-    Given two vertices point_a and point_b, each a 2-tuple,
-    return the square of the distance between them.
-    """
-    delta_x = point_a[0] - point_b[0]
-    delta_y = point_a[1] - point_b[1]
-    return delta_x * delta_x + delta_y * delta_y
-
-
 def vInitial_VF_A_Dx(v_final, acceleration, delta_x):
     """
     Kinematic calculation: Maximum allowed initial velocity to arrive at distance X
